@@ -21,7 +21,9 @@ package org.apache.maven.cli;
 import java.io.File;
 import java.io.FileNotFoundException;
 import java.io.FileOutputStream;
+import java.io.IOException;
 import java.io.PrintStream;
+import java.nio.charset.Charset;
 import java.nio.file.Files;
 import java.nio.file.Path;
 import java.nio.file.Paths;
@@ -31,6 +33,7 @@ import java.util.HashMap;
 import java.util.HashSet;
 import java.util.LinkedHashMap;
 import java.util.List;
+import java.util.ListIterator;
 import java.util.Locale;
 import java.util.Map;
 import java.util.Map.Entry;
@@ -46,6 +49,7 @@ import com.google.inject.AbstractModule;
 import org.apache.commons.cli.CommandLine;
 import org.apache.commons.cli.Option;
 import org.apache.commons.cli.ParseException;
+import org.apache.commons.cli.UnrecognizedOptionException;
 import org.apache.maven.InternalErrorException;
 import org.apache.maven.Maven;
 import org.apache.maven.building.FileSource;
@@ -56,6 +60,8 @@ import org.apache.maven.cli.configuration.SettingsXmlConfigurationProcessor;
 import org.apache.maven.cli.event.ExecutionEventLogger;
 import org.apache.maven.cli.internal.BootstrapCoreExtensionManager;
 import org.apache.maven.cli.internal.extension.model.CoreExtension;
+import org.apache.maven.cli.logging.Slf4jConfiguration;
+import org.apache.maven.cli.logging.Slf4jConfigurationFactory;
 import org.apache.maven.cli.transfer.QuietMavenTransferListener;
 import org.apache.maven.cli.transfer.Slf4jMavenTransferListener;
 import org.apache.maven.eventspy.internal.EventSpyDispatcher;
@@ -72,7 +78,6 @@ import org.apache.maven.extension.internal.CoreExtensionEntry;
 import org.apache.maven.lifecycle.LifecycleExecutionException;
 import org.apache.maven.model.building.ModelProcessor;
 import org.apache.maven.plugin.ExtensionRealmCache;
-import org.apache.maven.plugin.MavenPluginManager;
 import org.apache.maven.plugin.PluginArtifactsCache;
 import org.apache.maven.plugin.PluginRealmCache;
 import org.apache.maven.plugin.version.PluginVersionResolver;
@@ -93,7 +98,9 @@ import org.codehaus.plexus.PlexusContainer;
 import org.codehaus.plexus.classworlds.ClassWorld;
 import org.codehaus.plexus.classworlds.realm.ClassRealm;
 import org.codehaus.plexus.component.repository.exception.ComponentLookupException;
-import org.codehaus.plexus.util.StringUtils;
+import org.codehaus.plexus.interpolation.AbstractValueSource;
+import org.codehaus.plexus.interpolation.InterpolationException;
+import org.codehaus.plexus.interpolation.StringSearchInterpolator;
 import org.eclipse.aether.transfer.TransferListener;
 import org.mvndaemon.mvnd.cache.invalidating.InvalidatingExtensionRealmCache;
 import org.mvndaemon.mvnd.cache.invalidating.InvalidatingPluginArtifactsCache;
@@ -110,11 +117,12 @@ import org.mvndaemon.mvnd.logging.smart.BuildEventListener;
 import org.mvndaemon.mvnd.logging.smart.LoggingExecutionListener;
 import org.mvndaemon.mvnd.logging.smart.LoggingOutputStream;
 import org.mvndaemon.mvnd.plugin.CachingPluginVersionResolver;
-import org.mvndaemon.mvnd.plugin.CliMavenPluginManager;
 import org.mvndaemon.mvnd.transfer.DaemonMavenTransferListener;
 import org.slf4j.ILoggerFactory;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
+import org.slf4j.impl.MvndSimpleLogger;
+import org.slf4j.spi.LocationAwareLogger;
 import org.sonatype.plexus.components.sec.dispatcher.DefaultSecDispatcher;
 import org.sonatype.plexus.components.sec.dispatcher.SecDispatcher;
 
@@ -127,7 +135,7 @@ import static org.apache.maven.shared.utils.logging.MessageUtils.buffer;
  *
  * @author Jason van Zyl
  */
-public class DaemonMavenCli {
+public class DaemonMavenCli implements DaemonCli {
     public static final String LOCAL_REPO_PROPERTY = "maven.repo.local";
 
     public static final String MULTIMODULE_PROJECT_DIRECTORY = "maven.multiModuleProjectDirectory";
@@ -143,9 +151,14 @@ public class DaemonMavenCli {
 
     private static final String EXT_CLASS_PATH = "maven.ext.class.path";
 
-    private static final String EXTENSIONS_FILENAME = ".mvn/extensions.xml";
+    private static final String DOT_MVN = ".mvn";
 
-    private static final String MVN_MAVEN_CONFIG = ".mvn/maven.config";
+    private static final String UNABLE_TO_FIND_ROOT_PROJECT_MESSAGE = "Unable to find the root directory. Create a "
+            + DOT_MVN + " directory in the project root directory to identify it.";
+
+    private static final String EXTENSIONS_FILENAME = DOT_MVN + "/extensions.xml";
+
+    private static final String MVN_MAVEN_CONFIG = DOT_MVN + "/maven.config";
 
     public static final String STYLE_COLOR_PROPERTY = "style.color";
 
@@ -187,8 +200,7 @@ public class DaemonMavenCli {
         slf4jLogger = slf4jLoggerFactory.getLogger(this.getClass().getName());
         plexusLoggerManager = new Slf4jLoggerManager();
 
-        ClassLoader cl = Thread.currentThread().getContextClassLoader();
-        classWorld = new ClassWorld("plexus.core", cl);
+        this.classWorld = ((ClassRealm) Thread.currentThread().getContextClassLoader()).getWorld();
 
         container = container();
 
@@ -230,18 +242,20 @@ public class DaemonMavenCli {
             environment(cliRequest.workingDirectory, clientEnv);
             cli(cliRequest);
             properties(cliRequest);
-            help(cliRequest);
             logging(cliRequest);
+            informativeCommands(cliRequest);
+            version(cliRequest);
             container(cliRequest);
             configure(cliRequest, eventSpyDispatcher, configurationProcessors);
-            version(cliRequest);
             toolchains(cliRequest);
             populateRequest(cliRequest);
             encryption(cliRequest);
-            repository(cliRequest);
             return execute(cliRequest);
         } catch (ExitException e) {
             return e.exitCode;
+        } catch (UnrecognizedOptionException e) {
+            // pure user error, suppress stack trace
+            return 1;
         } finally {
             eventSpyDispatcher.close();
             System.setProperties(props);
@@ -257,32 +271,106 @@ public class DaemonMavenCli {
         }
 
         if (cliRequest.multiModuleProjectDirectory == null) {
-            buildEventListener.log(String.format("-D%s system property is not set.", MULTIMODULE_PROJECT_DIRECTORY));
-            throw new ExitException(1);
+            String basedirProperty = System.getProperty(MULTIMODULE_PROJECT_DIRECTORY);
+            if (basedirProperty == null) {
+                System.err.format("-D%s system property is not set.", MULTIMODULE_PROJECT_DIRECTORY);
+                throw new ExitException(1);
+            }
+            File basedir = new File(basedirProperty);
+            try {
+                cliRequest.multiModuleProjectDirectory = basedir.getCanonicalFile();
+            } catch (IOException e) {
+                cliRequest.multiModuleProjectDirectory = basedir.getAbsoluteFile();
+            }
+        }
+
+        // We need to locate the top level project which may be pointed at using
+        // the -f/--file option.  However, the command line isn't parsed yet, so
+        // we need to iterate through the args to find it and act upon it.
+        Path topDirectory = Paths.get(cliRequest.workingDirectory);
+        boolean isAltFile = false;
+        for (String arg : cliRequest.args) {
+            if (isAltFile) {
+                // this is the argument following -f/--file
+                Path path = topDirectory.resolve(arg);
+                if (Files.isDirectory(path)) {
+                    topDirectory = path;
+                } else if (Files.isRegularFile(path)) {
+                    topDirectory = path.getParent();
+                    if (!Files.isDirectory(topDirectory)) {
+                        System.err.println("Directory " + topDirectory
+                                + " extracted from the -f/--file command-line argument " + arg + " does not exist");
+                        throw new ExitException(1);
+                    }
+                } else {
+                    System.err.println(
+                            "POM file " + arg + " specified with the -f/--file command line argument does not exist");
+                    throw new ExitException(1);
+                }
+                break;
+            } else {
+                // Check if this is the -f/--file option
+                isAltFile = arg.equals(String.valueOf(CLIManager.ALTERNATE_POM_FILE)) || arg.equals("file");
+            }
+        }
+        topDirectory = getCanonicalPath(topDirectory);
+        cliRequest.topDirectory = topDirectory;
+        // We're very early in the process and we don't have the container set up yet,
+        // so we on searchAcceptableRootDirectory method to find us acceptable directory.
+        // The method may return null if nothing acceptable found.
+        cliRequest.rootDirectory = searchAcceptableRootDirectory(topDirectory);
+
+        //
+        // Make sure the Maven home directory is an absolute path to save us from confusion with say drive-relative
+        // Windows paths.
+        //
+        String mavenHome = System.getProperty("maven.home");
+
+        if (mavenHome != null) {
+            System.setProperty("maven.home", new File(mavenHome).getAbsolutePath());
+        }
+    }
+
+    protected boolean isAcceptableRootDirectory(Path path) {
+        return path != null && Files.isDirectory(path.resolve(DOT_MVN));
+    }
+
+    protected Path searchAcceptableRootDirectory(Path path) {
+        if (path == null) {
+            return null;
+        }
+        if (isAcceptableRootDirectory(path)) {
+            return path;
+        }
+        return searchAcceptableRootDirectory(path.getParent());
+    }
+
+    private static Path getCanonicalPath(Path path) {
+        try {
+            return path.toRealPath();
+        } catch (IOException e) {
+            return getCanonicalPath(path.getParent()).resolve(path.getFileName());
         }
-        System.setProperty("maven.multiModuleProjectDirectory", cliRequest.multiModuleProjectDirectory.toString());
     }
 
     void cli(CliRequest cliRequest) throws Exception {
         CLIManager cliManager = newCLIManager();
 
-        List<String> args = new ArrayList<>();
         CommandLine mavenConfig = null;
         try {
             File configFile = new File(cliRequest.multiModuleProjectDirectory, MVN_MAVEN_CONFIG);
 
             if (configFile.isFile()) {
-                for (String arg : new String(Files.readAllBytes(configFile.toPath())).split("\\s+")) {
-                    if (!arg.isEmpty()) {
-                        args.add(arg);
+                try (Stream<String> lines = Files.lines(configFile.toPath(), Charset.defaultCharset())) {
+                    String[] args = lines.filter(arg -> !arg.isEmpty() && !arg.startsWith("#"))
+                            .toArray(String[]::new);
+                    mavenConfig = cliManager.parse(args);
+                    List<?> unrecognized = mavenConfig.getArgList();
+                    if (!unrecognized.isEmpty()) {
+                        // This file can only contain options, not args (goals or phases)
+                        throw new ParseException("Unrecognized maven.config file entries: " + unrecognized);
                     }
                 }
-
-                mavenConfig = cliManager.parse(args.toArray(new String[0]));
-                List<?> unrecongized = mavenConfig.getArgList();
-                if (!unrecongized.isEmpty()) {
-                    throw new ParseException("Unrecognized maven.config entries: " + unrecongized);
-                }
             }
         } catch (ParseException e) {
             buildEventListener.log("Unable to parse maven.config: " + e.getMessage());
@@ -303,11 +391,20 @@ public class DaemonMavenCli {
         }
     }
 
-    private void help(CliRequest cliRequest) throws Exception {
+    private void informativeCommands(CliRequest cliRequest) throws Exception {
         if (cliRequest.commandLine.hasOption(CLIManager.HELP)) {
             buildEventListener.log(MvndHelpFormatter.displayHelp(newCLIManager()));
             throw new ExitException(0);
         }
+
+        if (cliRequest.commandLine.hasOption(CLIManager.VERSION)) {
+            if (cliRequest.commandLine.hasOption(CLIManager.QUIET)) {
+                buildEventListener.log(CLIReportingUtils.showVersionMinimal());
+            } else {
+                buildEventListener.log(CLIReportingUtils.showVersion());
+            }
+            throw new ExitException(0);
+        }
     }
 
     private CLIManager newCLIManager() {
@@ -363,15 +460,17 @@ public class DaemonMavenCli {
         cliRequest.quiet = !cliRequest.debug && cliRequest.commandLine.hasOption(CLIManager.QUIET);
         cliRequest.showErrors = cliRequest.debug || cliRequest.commandLine.hasOption(CLIManager.ERRORS);
 
-        ch.qos.logback.classic.Level level;
+        Slf4jConfiguration slf4jConfiguration = Slf4jConfigurationFactory.getConfiguration(slf4jLoggerFactory);
+
         if (cliRequest.debug) {
-            level = ch.qos.logback.classic.Level.DEBUG;
+            cliRequest.request.setLoggingLevel(MavenExecutionRequest.LOGGING_LEVEL_DEBUG);
+            slf4jConfiguration.setRootLoggerLevel(Slf4jConfiguration.Level.DEBUG);
         } else if (cliRequest.quiet) {
-            level = ch.qos.logback.classic.Level.WARN;
-        } else {
-            level = ch.qos.logback.classic.Level.INFO;
+            cliRequest.request.setLoggingLevel(MavenExecutionRequest.LOGGING_LEVEL_ERROR);
+            slf4jConfiguration.setRootLoggerLevel(Slf4jConfiguration.Level.ERROR);
         }
-        ((ch.qos.logback.classic.Logger) slf4jLoggerFactory.getLogger(Logger.ROOT_LOGGER_NAME)).setLevel(level);
+        // else fall back to default log level specified in conf
+        // see https://issues.apache.org/jira/browse/MNG-2570
 
         // LOG COLOR
         String styleColor = cliRequest.getUserProperties().getProperty(STYLE_COLOR_PROPERTY, "auto");
@@ -387,11 +486,6 @@ public class DaemonMavenCli {
             MessageUtils.setColorEnabled(false);
         }
 
-        // Workaround for https://github.com/apache/maven-mvnd/issues/39
-        final ch.qos.logback.classic.Logger mvndLogger =
-                (ch.qos.logback.classic.Logger) slf4jLoggerFactory.getLogger("org.mvndaemon.mvnd");
-        mvndLogger.setLevel(ch.qos.logback.classic.Level.toLevel(System.getProperty("mvnd.log.level"), null));
-
         // LOG STREAMS
         if (cliRequest.commandLine.hasOption(CLIManager.LOG_FILE)) {
             File logFile = new File(cliRequest.commandLine.getOptionValue(CLIManager.LOG_FILE));
@@ -411,23 +505,20 @@ public class DaemonMavenCli {
                 .asOptional()
                 .map(Boolean::parseBoolean)
                 .orElse(Boolean.FALSE)) {
-            ch.qos.logback.classic.Logger stdout =
-                    (ch.qos.logback.classic.Logger) slf4jLoggerFactory.getLogger("stdout");
-            ch.qos.logback.classic.Logger stderr =
-                    (ch.qos.logback.classic.Logger) slf4jLoggerFactory.getLogger("stderr");
-            stdout.setLevel(ch.qos.logback.classic.Level.INFO);
-            stderr.setLevel(ch.qos.logback.classic.Level.INFO);
+            MvndSimpleLogger stdout = (MvndSimpleLogger) slf4jLoggerFactory.getLogger("stdout");
+            MvndSimpleLogger stderr = (MvndSimpleLogger) slf4jLoggerFactory.getLogger("stderr");
+            stdout.setLogLevel(LocationAwareLogger.INFO_INT);
+            stderr.setLogLevel(LocationAwareLogger.INFO_INT);
             System.setOut(new LoggingOutputStream(s -> stdout.info("[stdout] " + s)).printStream());
             System.setErr(new LoggingOutputStream(s -> stderr.warn("[stderr] " + s)).printStream());
         }
+
+        slf4jConfiguration.activate();
     }
 
     private void version(CliRequest cliRequest) throws ExitException {
-        if (cliRequest.debug || cliRequest.commandLine.hasOption(CLIManager.VERSION)) {
+        if (cliRequest.debug || cliRequest.commandLine.hasOption(CLIManager.SHOW_VERSION)) {
             buildEventListener.log(CLIReportingUtils.showVersion());
-            if (cliRequest.commandLine.hasOption(CLIManager.VERSION)) {
-                throw new ExitException(0);
-            }
         }
     }
 
@@ -464,8 +555,77 @@ public class DaemonMavenCli {
 
     // Needed to make this method package visible to make writing a unit test possible
     // Maybe it's better to move some of those methods to separate class (SoC).
-    void properties(CliRequest cliRequest) {
-        populateProperties(cliRequest.commandLine, cliRequest.systemProperties, cliRequest.userProperties);
+    void properties(CliRequest cliRequest) throws ExitException {
+        try {
+            populateProperties(cliRequest, cliRequest.systemProperties, cliRequest.userProperties);
+
+            StringSearchInterpolator interpolator =
+                    createInterpolator(cliRequest, cliRequest.systemProperties, cliRequest.userProperties);
+            CommandLine.Builder commandLineBuilder = new CommandLine.Builder();
+            for (Option option : cliRequest.commandLine.getOptions()) {
+                if (!String.valueOf(CLIManager.SET_USER_PROPERTY).equals(option.getOpt())) {
+                    List<String> values = option.getValuesList();
+                    for (ListIterator<String> it = values.listIterator(); it.hasNext(); ) {
+                        it.set(interpolator.interpolate(it.next()));
+                    }
+                }
+                commandLineBuilder.addOption(option);
+            }
+            for (String arg : cliRequest.commandLine.getArgList()) {
+                commandLineBuilder.addArg(interpolator.interpolate(arg));
+            }
+            cliRequest.commandLine = commandLineBuilder.build();
+        } catch (InterpolationException e) {
+            String message = "ERROR: Could not interpolate properties and/or arguments: " + e.getMessage();
+            System.err.println(message);
+            throw new ExitException(1); // user error
+        } catch (MavenCli.IllegalUseOfUndefinedProperty e) {
+            String message = "ERROR: Illegal use of undefined property: " + e.property;
+            System.err.println(message);
+            if (cliRequest.rootDirectory == null) {
+                System.err.println();
+                System.err.println(UNABLE_TO_FIND_ROOT_PROJECT_MESSAGE);
+            }
+            throw new ExitException(1); // user error
+        }
+    }
+
+    protected static StringSearchInterpolator createInterpolator(CliRequest cliRequest, Properties... properties) {
+        StringSearchInterpolator interpolator = new StringSearchInterpolator();
+        interpolator.addValueSource(new AbstractValueSource(false) {
+            @Override
+            public Object getValue(String expression) {
+                if ("session.topDirectory".equals(expression)) {
+                    Path topDirectory = cliRequest.topDirectory;
+                    if (topDirectory != null) {
+                        return topDirectory.toString();
+                    } else {
+                        throw new MavenCli.IllegalUseOfUndefinedProperty(expression);
+                    }
+                } else if ("session.rootDirectory".equals(expression)) {
+                    Path rootDirectory = cliRequest.rootDirectory;
+                    if (rootDirectory != null) {
+                        return rootDirectory.toString();
+                    } else {
+                        throw new MavenCli.IllegalUseOfUndefinedProperty(expression);
+                    }
+                }
+                return null;
+            }
+        });
+        interpolator.addValueSource(new AbstractValueSource(false) {
+            @Override
+            public Object getValue(String expression) {
+                for (Properties props : properties) {
+                    Object val = props.getProperty(expression);
+                    if (val != null) {
+                        return val;
+                    }
+                }
+                return null;
+            }
+        });
+        return interpolator;
     }
 
     void container(CliRequest cliRequest) {
@@ -486,6 +646,7 @@ public class DaemonMavenCli {
 
         List<File> extClassPath = Stream.of(
                         Environment.MVND_EXT_CLASSPATH.asString().split(","))
+                .filter(s -> s != null && !s.isEmpty())
                 .map(File::new)
                 .collect(Collectors.toList());
 
@@ -536,7 +697,6 @@ public class DaemonMavenCli {
                 bind(PluginArtifactsCache.class).to(InvalidatingPluginArtifactsCache.class);
                 bind(PluginRealmCache.class).to(InvalidatingPluginRealmCache.class);
                 bind(ProjectArtifactsCache.class).to(InvalidatingProjectArtifactsCache.class);
-                bind(MavenPluginManager.class).to(CliMavenPluginManager.class);
                 bind(PluginVersionResolver.class).to(CachingPluginVersionResolver.class);
             }
         });
@@ -557,64 +717,56 @@ public class DaemonMavenCli {
     }
 
     private List<CoreExtensionEntry> loadCoreExtensions(
-            List<CoreExtension> extensions, ClassRealm containerRealm, Set<String> providedArtifacts) {
-        try {
-            if (extensions.isEmpty()) {
-                return Collections.emptyList();
-            }
-            ContainerConfiguration cc = new DefaultContainerConfiguration() //
-                    .setClassWorld(classWorld) //
-                    .setRealm(containerRealm) //
-                    .setClassPathScanning(PlexusConstants.SCANNING_INDEX) //
-                    .setAutoWiring(true) //
-                    .setJSR250Lifecycle(true) //
-                    .setName("maven");
+            List<CoreExtension> extensions, ClassRealm containerRealm, Set<String> providedArtifacts) throws Exception {
+        if (extensions.isEmpty()) {
+            return Collections.emptyList();
+        }
+        ContainerConfiguration cc = new DefaultContainerConfiguration() //
+                .setClassWorld(classWorld) //
+                .setRealm(containerRealm) //
+                .setClassPathScanning(PlexusConstants.SCANNING_INDEX) //
+                .setAutoWiring(true) //
+                .setJSR250Lifecycle(true) //
+                .setName("maven");
 
-            DefaultPlexusContainer container = new DefaultPlexusContainer(cc, new AbstractModule() {
-                @Override
-                protected void configure() {
-                    bind(ILoggerFactory.class).toInstance(slf4jLoggerFactory);
-                }
-            });
-            MavenExecutionRequestPopulator executionRequestPopulator = null;
-            try {
-                CliRequest cliRequest = new CliRequest(new String[0], classWorld);
-                cliRequest.commandLine = new CommandLine.Builder().build();
-                container.setLookupRealm(null);
-                container.setLoggerManager(plexusLoggerManager);
-                container.getLoggerManager().setThresholds(cliRequest.request.getLoggingLevel());
-                Thread.currentThread().setContextClassLoader(container.getContainerRealm());
-                executionRequestPopulator = container.lookup(MavenExecutionRequestPopulator.class);
-                final Map<String, ConfigurationProcessor> configurationProcessors =
-                        container.lookupMap(ConfigurationProcessor.class);
-                final EventSpyDispatcher eventSpyDispatcher = container.lookup(EventSpyDispatcher.class);
-                properties(cliRequest);
-                configure(cliRequest, eventSpyDispatcher, configurationProcessors);
-                LoggingExecutionListener executionListener = container.lookup(LoggingExecutionListener.class);
-                populateRequest(
-                        cliRequest,
-                        cliRequest.request,
-                        slf4jLogger,
-                        eventSpyDispatcher,
-                        container.lookup(ModelProcessor.class),
-                        createTransferListener(cliRequest),
-                        buildEventListener,
-                        executionListener);
-                executionRequestPopulator.populateDefaults(cliRequest.request);
-                BootstrapCoreExtensionManager resolver = container.lookup(BootstrapCoreExtensionManager.class);
-                return Collections.unmodifiableList(
-                        resolver.loadCoreExtensions(cliRequest.request, providedArtifacts, extensions));
-            } finally {
-                executionRequestPopulator = null;
-                container.dispose();
+        DefaultPlexusContainer container = new DefaultPlexusContainer(cc, new AbstractModule() {
+            @Override
+            protected void configure() {
+                bind(ILoggerFactory.class).toInstance(slf4jLoggerFactory);
             }
-        } catch (RuntimeException e) {
-            // runtime exceptions are most likely bugs in maven, let them bubble up to the user
-            throw e;
-        } catch (Exception e) {
-            slf4jLogger.warn("Failed to load extensions descriptor {}: {}", extensions, e.getMessage());
+        });
+        MavenExecutionRequestPopulator executionRequestPopulator = null;
+        try {
+            CliRequest cliRequest = new CliRequest(new String[0], classWorld);
+            cliRequest.commandLine = new CommandLine.Builder().build();
+            container.setLookupRealm(null);
+            container.setLoggerManager(plexusLoggerManager);
+            container.getLoggerManager().setThresholds(cliRequest.request.getLoggingLevel());
+            Thread.currentThread().setContextClassLoader(container.getContainerRealm());
+            executionRequestPopulator = container.lookup(MavenExecutionRequestPopulator.class);
+            final Map<String, ConfigurationProcessor> configurationProcessors =
+                    container.lookupMap(ConfigurationProcessor.class);
+            final EventSpyDispatcher eventSpyDispatcher = container.lookup(EventSpyDispatcher.class);
+            properties(cliRequest);
+            configure(cliRequest, eventSpyDispatcher, configurationProcessors);
+            LoggingExecutionListener executionListener = container.lookup(LoggingExecutionListener.class);
+            populateRequest(
+                    cliRequest,
+                    cliRequest.request,
+                    slf4jLogger,
+                    eventSpyDispatcher,
+                    container.lookup(ModelProcessor.class),
+                    createTransferListener(cliRequest),
+                    buildEventListener,
+                    executionListener);
+            executionRequestPopulator.populateDefaults(cliRequest.request);
+            BootstrapCoreExtensionManager resolver = container.lookup(BootstrapCoreExtensionManager.class);
+            return Collections.unmodifiableList(
+                    resolver.loadCoreExtensions(cliRequest.request, providedArtifacts, extensions));
+        } finally {
+            executionRequestPopulator = null;
+            container.dispose();
         }
-        return Collections.emptyList();
     }
 
     private ClassRealm setupContainerRealm(
@@ -658,12 +810,10 @@ public class DaemonMavenCli {
 
         List<File> jars = new ArrayList<>();
 
-        if (StringUtils.isNotEmpty(extClassPath)) {
-            for (String jar : StringUtils.split(extClassPath, File.pathSeparator)) {
+        if (extClassPath != null) {
+            for (String jar : extClassPath.split(File.pathSeparator)) {
                 File file = resolveFile(new File(jar), cliRequest.workingDirectory);
-
                 slf4jLogger.debug("  Included {}", file);
-
                 jars.add(file);
             }
         }
@@ -682,13 +832,6 @@ public class DaemonMavenCli {
         }
     }
 
-    private void repository(CliRequest cliRequest) throws Exception {
-        if (cliRequest.commandLine.hasOption(CLIManager.LEGACY_LOCAL_REPOSITORY)
-                || Boolean.getBoolean("maven.legacyLocalRepo")) {
-            cliRequest.request.setUseLegacyLocalRepository(true);
-        }
-    }
-
     private void environment(String workingDir, Map<String, String> clientEnv) {
         EnvHelper.environment(workingDir, clientEnv);
     }
@@ -832,19 +975,11 @@ public class DaemonMavenCli {
 
     private void logSummary(
             ExceptionSummary summary, Map<String, String> references, String indent, boolean showErrors) {
-        String referenceKey = "";
-
-        if (StringUtils.isNotEmpty(summary.getReference())) {
-            referenceKey = references.get(summary.getReference());
-            if (referenceKey == null) {
-                referenceKey = "[Help " + (references.size() + 1) + "]";
-                references.put(summary.getReference(), referenceKey);
-            }
-        }
-
         String msg = summary.getMessage();
 
-        if (StringUtils.isNotEmpty(referenceKey)) {
+        if (!summary.getReference().isEmpty()) {
+            String referenceKey =
+                    references.computeIfAbsent(summary.getReference(), k -> "[Help " + (references.size() + 1) + "]");
             if (msg.indexOf('\n') < 0) {
                 msg += " -> " + buffer().strong(referenceKey);
             } else {
@@ -1250,6 +1385,7 @@ public class DaemonMavenCli {
 
         request.setCacheNotFound(true);
         request.setCacheTransferError(false);
+        request.setIgnoreTransitiveRepositories(commandLine.hasOption(CLIManager.IGNORE_TRANSITIVE_REPOSITORIES));
 
         //
         // Builder, concurrency and parallelism
@@ -1305,7 +1441,8 @@ public class DaemonMavenCli {
     // System properties handling
     // ----------------------------------------------------------------------
 
-    static void populateProperties(CommandLine commandLine, Properties systemProperties, Properties userProperties) {
+    static void populateProperties(CliRequest cliRequest, Properties systemProperties, Properties userProperties)
+            throws InterpolationException {
         addEnvVars(systemProperties);
 
         // ----------------------------------------------------------------------
@@ -1314,18 +1451,26 @@ public class DaemonMavenCli {
         // are most dominant.
         // ----------------------------------------------------------------------
 
-        if (commandLine.hasOption(CLIManager.SET_SYSTEM_PROPERTY)) {
-            String[] defStrs = commandLine.getOptionValues(CLIManager.SET_SYSTEM_PROPERTY);
+        Properties cliProperties = new Properties();
+        if (cliRequest.commandLine.hasOption(CLIManager.SET_SYSTEM_PROPERTY)) {
+            String[] defStrs = cliRequest.commandLine.getOptionValues(CLIManager.SET_SYSTEM_PROPERTY);
 
             if (defStrs != null) {
                 for (String defStr : defStrs) {
-                    setCliProperty(defStr, userProperties);
+                    setCliProperty(defStr, cliProperties);
                 }
             }
         }
 
         SystemProperties.addSystemProperties(systemProperties);
 
+        StringSearchInterpolator interpolator = createInterpolator(cliRequest, cliProperties, systemProperties);
+        for (Map.Entry<Object, Object> e : cliProperties.entrySet()) {
+            String name = (String) e.getKey();
+            String value = interpolator.interpolate((String) e.getValue());
+            userProperties.setProperty(name, value);
+        }
+
         // ----------------------------------------------------------------------
         // Properties containing info about the currently running version of Maven
         // These override any corresponding properties set on the command line
